%%

%class Flexer
%unicode
/*%debug*/
%int
%line
%column

%{
	Nonterm nonterm = new Nonterm();
	Alphabet alphabet = new Alphabet();
	Rules rules = new Rules();
	String firstState; 
	
	String rule1;
	static int syn = 0;
%}
lineTer = \r|\n|\r\n
WhiteSpace = {lineTer} | [ \t\f]
letter = [a-d]|[f-z]
special = "$"|"&"|"!"|"#"|"-"|"."|"/"|":"|";"|"<"|">"|"="|"@"|"["|"]"|"^"|"`"|"~"|"+"|"?"|"*"|"\"|"""|"|"|"˜"|"‘"|"´"|"%"

terminal = {letter}|{special}|[:digit:]

nonterminal = [A-Z]
non_terminal = {nonterminal}|{terminal}

replacement = e|{non_terminal}+

%state BRA1 NON1 SEP VIR1 BRA2 ALP SEP2 VIR2 BRA3 SEP3 RUL1 RUL2 AUXVIR VIR3 FIN LBR
/*
  **BRA1:   reads first curly bracket
  **NON1:   reads each nonterminal
  **SEP:    reads separators
  **VIR1:   reads the comma between the nonterminals and the alphabet
  **BRA2:   reads first curly bracket of the alphabet
  **ALP:    reads the alphabet
  **SEP2:   reads separators of the alphabet
  **VIR2:   reads the comma between the alphabet and the rules
  **BRA3:   reads first curly bracket of the rules
  **SEP3:   reads separators of the rules
  **RUL1:   reads the nonterminal of the rule
  **RUL2:   reads the replacement of the rule
  **AUXVIR: reads the comma between rules or the last curly bracket
  **VIR3:   reads the comma between rules and the final state
  **FIN:    reads the final state
  **LBR:    reads the last curly bracket
*/

%%
{WhiteSpace} {/*skip whitespece*/}

<YYINITIAL>"(" { /* read first bracket */
	yybegin(BRA1);
}

<BRA1>"{" { /* read first curly bracket */
	yybegin(SEP);
}

<NON1>{non_terminal} { /* reads every nonterminal */
	nonterm.addLetter(yytext());
	yybegin(SEP);
}


<SEP>{ /* read separator*/
	{non_terminal} 		{nonterm.addLetter(yytext()); yybegin(SEP);}
	"," 			{yybegin(NON1);}
	"}" 			{yybegin(VIR1);}
}

<VIR1>"," { /*reads the comma between the nonterminals and the alphabet */
	yybegin(BRA2);
}

<BRA2>"{" { /* read first curly bracket */
	yybegin(SEP2);
}

<ALP>{terminal} { /*reads the alphabet*/
	alphabet.addLetter(yytext());
	yybegin(SEP2);
}

<SEP2>{ /* read separator*/
	{terminal} 		{alphabet.addLetter(yytext()); yybegin(SEP2);}
	"," 			{yybegin(ALP);}
	"}" 			{yybegin(VIR2);}
}

<VIR2>"," { /*reads the comma between the alhabet and the production rules */
	yybegin(BRA3);
}

<BRA3>"{" { /* read first curly bracket */
	yybegin(SEP3);	
}

<SEP3>{ /* reads separators between rules */
	"(" 			{yybegin(RUL1);}
	"," 			{yybegin(RUL2);}
	")" 			{yybegin(AUXVIR);}
	"}" 			{yybegin(VIR3);}
}

<RUL1>{nonterminal} { /* reads the nonterminal of the rule */
	rule1 = yytext();
	yybegin(SEP3);
}

<RUL2>{replacement} { /* reads the replacement */
	rules.addRule(rule1, yytext());
	yybegin(SEP3);
}

<AUXVIR>{ /* reads the comma between the rules or the last curly bracket */
	"," 			{yybegin(SEP3);}
	"}" 			{yybegin(VIR3);}
}

<VIR3>"," { /* reads the comma between the rules and the final state*/
	yybegin(FIN);
}

<FIN>{non_terminal} { /* reads the final state */
	firstState = yytext();
	yybegin(LBR);
}

<LBR>")" { /*reads the last bracket */
}

. {syn=1;}

